<?php
//////////////////////////////////////////////////////////////////////////////////
//    					 copyright (c) Arfooo Annuaire                          //
//   				 by Hocine Guillaume (c) 2007 - 2008                        //
//       					http://www.arfooo.com/                              //
//    Licence Creative Commons http://creativecommons.org/licenses/by/2.0/fr/   //
//////////////////////////////////////////////////////////////////////////////////

class WebmasterController extends Controller
{    
    /**
    * Initailize controller set access privileges
    */
    function init()
    {        
        $this->acl->allow('webmaster', $this->name, "*");
        
        $this->acl->allow('guest', $this->name, "logIn");
        $this->acl->allow('guest', $this->name, "lostPassword");
        $this->acl->allow('guest', $this->name, "isEmailAcceptable");
        $this->acl->allow('guest', $this->name, "register");
        $this->acl->allow('guest', $this->name, "submit1");
        $this->acl->allow('guest', $this->name, "submit2");
        $this->acl->allow('guest', $this->name, "submit3");
        $this->acl->allow('guest', $this->name, "getBackLinkData");
         
        if(!$this->acl->isAllowed($this->session->get("role"), $this->name, $this->action))
        {
            $this->redirect($this->moduleLink("logIn"));  
        }
    }
    
    /**
    * Check that during webmaster inscription email can be acceptabe (not banned and busy)
    * @return mixed AjaxJson 
    */
    function isEmailAcceptableAction()
    {                    
        $status = "ok";
        $email = $this->request->email;
             
        $c = new Criteria();
        $c->add("email", $email);
        
        //check is email busy
        if($this->user->getCount($c))
        {
            $status = "busy";
        }
        else
        {
            //check is email banned
            if($this->bannedEmail->isBanned($email))
            {
                $status = "banned";
            }
        } 

        $this->set("status", $status);
        $this->viewClass = "JsonView";
    }
    
    /**
    * Validate webmaster data
    * @return string message or false
    */
    function validateRegistration()
    {
        if(Config::get("captchaEnabledOnWebmasterRegistration") && !$this->captchaCode->validatePublicAndPrivateCodes($this->request))
            return "You did not guess the security code. Try again with a new one.";
                                                                                                                               
        if(empty($this->request->email) || !preg_match("#^(([A-Za-z0-9_])|([\-\.]))+\@(([A-Za-z0-9_])|([\-\.]))+\.([A-Za-z]{2,4})$#", $this->request->email))
            return "Missed email";

        $c = new Criteria();
        $c->add("email", $this->request->email);
        
        if($this->user->getCount($c) > 0)
            return "Busy email";
            
        if($this->bannedEmail->isBanned($this->request->email))
            return "This email is banned.";
            
        if($this->bannedIp->isBanned($this->request->getIp()))
           return "Your IP is banned.";  
            
        if(empty($this->request->password))
            return "Empty password";
            
        return false;
        
    }
    
    function saveNewPasswordAction()
    {
        if(!empty($this->request->newPassword)
        && $this->request->newPassword == $this->request->repeatedNewPassword)
        {
            $this->user->updateByPk(array("password" => md5($this->request->newPassword)), $this->userId);
        }
        
        $this->redirect($this->moduleLink());    
    } 
    
    function changePasswordAction()
    {
    
    }
    
    /**
    * Register webmaster from ajax query 
    */
    function registerAction()
    {
        $this->viewClass = "JsonView";
        
        $errorMessage = $this->validateRegistration();
        
        if(!$errorMessage)
        {
            $user = new UserRecord();
            $user->email = $this->request->email;
            $user->password = md5($this->request->password);
            $user->role = 'webmaster';
            $user->save();
            
            //everything goes okey set
            $this->set("registered", true); 
        }
        else
        {
            //something is wrong with webmasterData
            $this->set("message", _t($errorMessage)); 
            $this->set("registered", false); 
        }

    }
    
    /**
    * Redirect to managment panel
    */
    function indexAction()
    {
        $this->redirect($this->moduleLink("manage"));
    }
    
    /**
    * Method to saving new/edited sites
    */
    function saveNewWebsiteAction()
    {
        $this->viewClass = "JsonView";
        
        //copy currect webmaster email to be in site row
        $this->request->webmasterEmail = $this->session->get("email"); 

        // if we edit site
        if(!empty($this->request->siteId))
        {
            $c = new Criteria();
            $c->add("siteId", $this->request->siteId);
            $c->add("webmasterId", $this->userId);
            $site = $this->site->find($c);
            
            //this site doesn't exists
            if(empty($site))
            {
                $this->set("status", "error");
                $this->set("message", _t("This site doesn't exists"));
                return;
            }
            
            //validate edited site and for some things compare with old state
            $errorMessage = $this->site->validate($this->request, $site);
        }
        else
        {            
            //validate new site 
            $errorMessage = $this->site->validate($this->request);
            
            //if everything okey create new SiteRecord
            if(!$errorMessage)
            {
                $site = new SiteRecord();
                $site->webmasterId = $this->userId;
            }
        }
 
        if($errorMessage)
        {
            //set status to js part about that something is wrong with site data
            $this->set("status", "error");
            
            //set error message what is wrong
            $this->set("message", _t($errorMessage));
            return;
        }

        //values which will be retrieved from post data
        $fields = array("categoryId", "siteTitle", "url", "rssTitle", "rssFeedOfSite", "returnBond",
                        "description", "proposalForCategory", "proposalForKeywords", "webmasterEmail");
                        
        if(Config::get("countryFlagsEnabled"))
        {
            array_push($fields, "countryCode");
        }
        
        if(Config::get("companyInfoEnabled"))
        {
            array_push($fields, "address", "zipCode", "city", "country", "phoneNumber", "faxNumber");
        }
                        
        //fillling siteRecord with post data                
        $site->fromArray($this->request->getArray($fields));
        
        $site->description = substr($site->description, 0, Config::get('siteDescriptionMaxLength'));       
        $site->status = (Config::get('automaticSiteValidation') == 1) ? 'validated' : 'waiting';
        $site->save();

        //store keywords
        if(!empty($this->request->keywords))
        {
            $this->keywordsOfSite->storeKeywords($site->siteId, $this->request->keywords);
        }

        //send email to admin that site was submitted/edited 
        if(Config::get('sendEmailsOnInscriptionAndApproval')) 
        {
            Mailer::getInstance()->sendAdminNotification($site);
        }
        
        //send notifications to webmaster that his site was saved to database
        if(Config::get('informWebmastersForAdminDecisions')) 
        {
            Mailer::getInstance()->sendWebmasterNotification($site, $this->session->get("email"));
        }
        
        //set status to js part about site proccessing
        $this->set("status", "ok"); 
                    
        if(!empty($this->request->siteId))
        {
            $this->set("message", _t("Your site was submitted again."));
        }
        else
        {
            //inform js to refresh page
            $this->set("refresh", true);
            $this->set("message", _t("The site was successfully submitted."));
        }
 
    }
    
    function getBackLinkDataAction()
    {
        $this->viewClass = "JsonView";
        
        $categoryId = $this->request->categoryId;
        $category = $this->category->findByPk($categoryId);
        $categoryBackLinkCode = '';
        
        if(!empty($category))
        {
            $backLinkUrl = Config::get("siteRootUrl"). FrontController::getRewrittedUrl('/site/category/'.$category->categoryId.'/'.str_replace("/", " ", $category->name).'/1', false);
            $categoryBackLinkCode = '<a href="'.$backLinkUrl.'"> '.Config::get("siteTitle").' - '.$category->name.'</a>';        
        }
        
        $this->set("categoryBackLinkCode", $categoryBackLinkCode);             
    }
    
    /**
    * Get edited site data
    */
    function getWebsiteDataAction()
    {
        $this->viewClass = "JsonView"; 
        
        $siteId = $this->request->siteId;
        $site = $this->site->findByPk($siteId);
        
        //if site don't exists
        if(empty($site))
        {
            return;
        }
        
        //find keywords selected for this site
        $c = new Criteria();
        $c->add("siteId", $siteId);
        $keywordIds = array();
        
        foreach($this->keywordsOfSite->findAll($c) as $keyword)
        {
            $keywordIds[] = $keyword['keywordId'];
        }
        
        //set keywords Ids for this site
        $site->keywordIds = $keywordIds;   
        $this->set($site->toArray());
    }
    
    /**
    * Get meta data for specified url 
    */
    function getMetaDataAction()
    {
        $httpClient = new HttpClient();
        $metaData = $httpClient->getMetaValues($this->request->url);
        $this->set($metaData);
        $this->viewClass = "JsonView";    
    }
    
    /**
    * Display edit form
    */
    function editSiteAction($siteId)
    {
        $this->submitWebsiteAction();
        $this->set("siteId", intval($siteId));                                            
        $this->set("websiteDataUrl", $this->moduleLink("getWebsiteData"));
    }
    
    /**
    * Form to submit/edit sites
    */
    function submitWebsiteAction()
    {
        //set selected category from cookie
        $this->set("selectedCategoryId", isset($_COOKIE['lastVisitedCategoryId']) ? intval($_COOKIE['lastVisitedCategoryId']) : ""); 
        $this->set("allKeywordsList", $this->keyword->generateSortedList());
        $this->set("selectCategories", $this->category->createOptionsList(true));  
        $this->set("metaDataUrl", $this->moduleLink("getMetaData"));
        
        require(CODE_ROOT_DIR."config/flags.php");
        $this->set("countryFlags", $countryFlags);
    }
    
    /**
    * Managment panel list of all sites which can be managed 
    */
    function manageAction()
    {
        $c = new Criteria();
        $c->add("webmasterId", $this->userId);
        $c->add("status", "banned", "<>");
        $c->addOrder("status, siteTitle");
        
        $sites = $this->site->findAll($c);
        $this->set("sites", $sites);
    }
    
    /**
    * Delete site
    */
    function deleteSiteAction($site_id)
    {
        $site = $this->site->findByPk($site_id);
        if($site)
        {
            $site->del();
        }
        
        $this->redirect($this->moduleLink("manage"));
    }
    
    /**
    * Login and inscription form 
    */
    function logInAction()
    {
        //if form was posted
        if(isset($this->request->email) && isset($this->request->password))
        {
            $email = $this->request->email;
            
            //check that user haven't got banned ip/email and password is correct
            if(!$this->bannedEmail->isBanned($email)
            && !$this->bannedIp->isBanned($this->request->getIp())
            && $this->session->loginUser($email, md5($this->request->password), "email", "webmaster"))
            {
                //if yes redirect to managment panel
                $this->redirect($this->moduleLink("manage"));
            }
            else
            {
                //if no set error message
                $this->set("loginError", 1);
            }
        }

        $this->set("directoryCondition", $this->getDirectoryCondition());  
    }
    
    /**
    * Retrieve html content from db which describe directory condition usage
    */
    private function getDirectoryCondition()
    {
        $cache = Cacher::getInstance();
        
        if(($message = $cache->load("directoryCondition")) === null)
        {
            $message = $this->customMessage->findByPk("directoryCondition");
            $cache->save($message, null, null, array("custommessage"));
        }
        
        return $message;      
    }
    
    /**
    * Logoff webmaster 
    */
    function logOffAction()
    {
        $this->session->destroy();
        $this->redirect($this->moduleLink());
    }
    
    /**
    * Lost password form, generation and sending  
    */
    function lostPasswordAction()
    {
        if(empty($this->request->email))return;
        
        //get user which have this email and role webmaster
        $c = new Criteria();
        $c->add("email", $this->request->email);
        $c->add("role", "webmaster");
        $user = $this->user->find($c);
        
        if(empty($user))return;

        $newPassword = $user->generateNewPassword();

        Mailer::getInstance()->sendLostPassword($user->email, $newPassword);

        $this->redirect($this->moduleLink("logIn"));
    }
        
    /**
    * Step 1 submitting site when webmasters must be registered switched to OFF
    */
    function submit1Action()
    {
        if(!Config::get('inscriptionsOfSitesEnabled'))
            die(_t('Inscriptions are currently disabled. Try again later.'));

        if(Config::get('registrationOfWebmastersEnabled'))
            die();

        if(!Config::get('metaTagScannerEnabled'))
            $this->redirect(FrontController::getRewrittedUrl("/webmaster/submit2"));
            
        $this->set("directoryCondition", $this->getDirectoryCondition());
    }
    
    /**
    * Step 2 submitting site when webmasters must be registered switched to OFF
    */
    function submit2Action()
    {
        if(isset($this->request->url))
        {
            $url = $this->request->url;
            $this->set("url", $url);
            
            //if user don't skip meta scanner with form button
            if(isset($this->request->withMetaTagsScanner))
            {
                $httpClient = new HttpClient();
                $metaData = $httpClient->getMetaValues($url);
                $this->set("metaData", $metaData);
            }
        }
             
        //set selected category from cookie value
        $this->set("selectedCategoryId", isset($_COOKIE['lastVisitedCategoryId']) ? intval($_COOKIE['lastVisitedCategoryId']) : "");
        $this->set("allKeywordsList", $this->keyword->generateSortedList());
        $this->set("selectCategories", $this->category->createOptionsList(true));
        require(CODE_ROOT_DIR."config/flags.php");
        $this->set("countryFlags", $countryFlags);         
    }
    
    /**
    * Step 3 submitting site when webmasters must be registered switched to OFF
    */
    function submit3Action()
    {        
        //valid new site data
        $errorMessage = $this->site->validate($this->request);
        
        //checking captcha
        if(Config::get('captchaEnabledOnSiteInscription') 
            && !$this->captchaCode->validatePublicAndPrivateCodes($this->request))
            $errorMessage = 'You did not guess the security code.';
        
        //if something is wrong
        if($errorMessage)
        {
            $this->set("errorMessage", _t($errorMessage));
            return;
        }

        $site = new SiteRecord();

        $fields = array("categoryId", "siteTitle", "webmasterName", "webmasterEmail", "url", "rssTitle", "rssFeedOfSite", "returnBond",
                        "description", "proposalForCategory", "proposalForKeywords");
                        
        if(Config::get("countryFlagsEnabled"))
        {
            array_push($fields, "countryCode");
        }
        
        if(Config::get("companyInfoEnabled"))
        {
            array_push($fields, "address", "zipCode", "city", "country", "phoneNumber", "faxNumber");
        }
                        
        $site->fromArray($this->request->getArray($fields));
        
        $site->description = substr($site->description, 0, Config::get('siteDescriptionMaxLength'));       
        $site->status = (Config::get('automaticSiteValidation') == 1) ? 'validated' : 'waiting'; 
        $site->webmasterId = 0;
        $site->save();

        //store keywords
        if(!empty($this->request->keywords))
        {
            $this->keywordsOfSite->storeKeywords($site->siteId, $this->request->keywords);
        }

        //send email to admin that site was submitted/edited
        if(Config::get('sendEmailsOnInscriptionAndApproval')) 
        {
            Mailer::getInstance()->sendAdminNotification($site);
        }
        
        //send notifications to webmaster that his site was saved to database   
        if(Config::get('informWebmastersForAdminDecisions')) 
        {
            Mailer::getInstance()->sendWebmasterNotification($site, $site->webmasterEmail);
        }
     
    }  
}
?>